"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const Debug = require("debug");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const debug = Debug('ionic:cli:commands:doctor:base');
class DoctorCommand extends command_1.Command {
    getRegistry() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this._registry) {
                this._registry = yield this.env.project.getAilmentRegistry(this.env);
            }
            return this._registry;
        });
    }
    detectAilments() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const registry = yield this.getRegistry();
            const config = yield this.env.config.load();
            let count = 0;
            const isLoggedIn = yield this.env.session.isLoggedIn();
            if (!isLoggedIn) {
                this.env.log.warn(`For best results, please make sure you're logged in to Ionic.\nSome issues can't be detected without authentication. Run:\n\n    ${chalk_1.default.green('ionic login')}`);
            }
            this.env.tasks.next('Detecting issues');
            const ailments = yield Promise.all(registry.ailments.map((ailment) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let detected = false;
                try {
                    debug(`Detecting ${chalk_1.default.bold(ailment.id)}`);
                    detected = yield ailment.detected();
                }
                catch (e) {
                    this.env.log.error(`Error while checking ${chalk_1.default.bold(ailment.id)}:\n` +
                        `${chalk_1.default.red(e.stack ? e.stack : e)}`);
                }
                count++;
                this.env.tasks.updateMsg(`Detecting issues: ${chalk_1.default.bold(`${count} / ${registry.ailments.length}`)} complete`);
                return [ailment, detected];
            })));
            this.env.tasks.updateMsg(`Detecting issues: ${chalk_1.default.bold(`${registry.ailments.length} / ${registry.ailments.length}`)} complete`);
            const detectedAilments = ailments
                .filter(([, detected]) => detected)
                .map(([ailment]) => ailment)
                .filter(ailment => {
                const issueConfig = config.doctor.issues[ailment.id];
                if (issueConfig && issueConfig.ignored === true) {
                    return false;
                }
                return true;
            });
            this.env.tasks.end();
            return detectedAilments;
        });
    }
    detectTreatableAilments() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ailments = yield this.detectAilments();
            return ailments.filter((ailment) => cli_utils_1.isTreatableAilment(ailment));
        });
    }
}
exports.DoctorCommand = DoctorCommand;
